#  This file is a part of the LinkAhead Project.
#
#  Copyright (C) 2024 Henrik tom Wörden (h.tomwoerden@indiscale.com)
#  Copyright (C) 2024 IndiScale GmbH (info@indiscale.com)
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program. If not, see <https://www.gnu.org/licenses/>.

"""
basic integration test of the loan workflow
"""

from pytest import fixture
import json
import linkahead as db
import re
from tempfile import NamedTemporaryFile
from linkahead.utils.server_side_scripting import run_server_side_script
from loan.box_loan import (BORROWER, BOX, COMMENT, DESTINATION, EXHAUST_CONTENTS, LENT,
                      F_CURRENT_LOCATION,
                      EXPECTED_RETURN, F_BOX, F_COMMENT, F_DESTINATION,PERSON, LOAN_ACCEPTED,
                      F_EMAIL, F_EXHAUST_CONTENTS, F_EXPECTED_RETURN_DATE, EMAIL, F_LOAN,
                      F_FIRST_NAME, F_LAST_NAME, FIRST_NAME, LAST_NAME, LOAN, LOCATION, RETURNED,
                      RETURN_ACCEPTED,
                      LOAN_REQUESTED, assert_date_in_future, CONTENT, RETURNLOCATION,
                      assert_key_in_data, get_box, insert_or_update_person, main, RETURN_REQUESTED,
                      send_loan_request_mail)


TESTLOANCOMMENT = 'This is a test'
TESTRETURNCOMMENT = 'This is a return test'
TESTBOXNUMBER = '0123 TEST'
TESTDESTINATION = 'TEST DEST'
TESTCURRENTDESTINATION = 'TEST CUR DEST'
TESTBORROWEREMAIL = 'test@example.com'
TESTRETURNDATE="2025-03-04"
TESTFN = "Alice"
TESTLN = "Wunderland"


def save_dict_to_jsonfile(data):
    tmp = NamedTemporaryFile(delete=False, suffix=".json")
    tmp.close()
    with open(tmp.name, "w") as fi:
        json.dump(data, fi)
    return tmp.name

def delete_stuff():
    to_be_deleted = db.Container()
    to_be_deleted.extend(db.execute_query(f"FIND loan with {COMMENT.name}='{TESTLOANCOMMENT}'"))
    to_be_deleted.extend(db.execute_query(f"FIND '{TESTDESTINATION}'"))
    to_be_deleted.extend(db.execute_query(f"FIND '{TESTCURRENTDESTINATION}'"))
    to_be_deleted.extend(db.execute_query(f"FIND '{TESTBOXNUMBER}'"))
    to_be_deleted.extend(db.execute_query(f"FIND person with {EMAIL.name}='{TESTBORROWEREMAIL}'"))
    if len(to_be_deleted)>0:
        to_be_deleted.delete()

@fixture(autouse=True)
def prepare():
    delete_stuff()
    db.Record(name=TESTBOXNUMBER).add_parent(BOX.name).insert()
    db.Record(name=TESTDESTINATION).add_parent(LOCATION.name).insert()
    db.Record(name=TESTCURRENTDESTINATION).add_parent(LOCATION.name).insert()
    (db.Record().add_parent(PERSON.name).add_property(EMAIL.name, TESTBORROWEREMAIL)
                .add_property(FIRST_NAME.name, TESTFN)
                .add_property(LAST_NAME.name, TESTLN).insert())
    yield
    delete_stuff()


def test_request_loan():

    ##### request loan #####
    data = {}
    data[F_EXHAUST_CONTENTS] = False
    data[F_EXPECTED_RETURN_DATE]= "2042-02-02"
    data[F_BOX]=db.utils.get_entity.get_entity_by_name(TESTBOXNUMBER).id
    data[F_COMMENT]=TESTLOANCOMMENT
    data[F_DESTINATION]=db.utils.get_entity.get_entity_by_name(TESTDESTINATION).id
    data[F_EMAIL]=TESTBORROWEREMAIL
    data[F_LAST_NAME]=TESTLN
    data[F_FIRST_NAME]=TESTFN

    response = run_server_side_script("loan_management/request_loan.py",
                                    #"pos0",
                                    #option1="val1",
                                    files={"-p0": save_dict_to_jsonfile(data)},
                                    **{"auth-token":db.get_connection()._authenticator.auth_token})
    assert response.stderr is None
    assert response.code == 0
    loan = db.execute_query(f"FIND loan with {COMMENT.name}='{TESTLOANCOMMENT}'", unique=True)
    assert loan.get_property(f"{BOX.name}").value == data[F_BOX]
    assert loan.get_property(f"{LOAN_REQUESTED.name}").value.startswith("20")
    assert loan.get_property(f"{BORROWER.name}").value == db.execute_query(
        f"FIND {PERSON.name} with {EMAIL.name}='{TESTBORROWEREMAIL}'", unique=True).id
    assert loan.get_property(f"{EXPECTED_RETURN.name}").value == data[F_EXPECTED_RETURN_DATE]
    assert loan.get_property(f"{EXHAUST_CONTENTS.name}").value == data[F_EXHAUST_CONTENTS]
    assert loan.get_property(f"{DESTINATION.name}").value == data[F_DESTINATION]
    assert loan.get_property(f"{LOAN_ACCEPTED.name}") is None
    assert loan.get_property(f"{LENT.name}") is None

    ##### accept loan #####
    data = {}
    data[F_LOAN] = loan.id
    response = run_server_side_script("loan_management/accept_loan_request.py",
                                    #"pos0",
                                    #option1="val1",
                                    files={"-p0": save_dict_to_jsonfile(data)},
                                    **{"auth-token":db.get_connection()._authenticator.auth_token})
    assert response.stderr is None, response.stderr
    assert response.code == 0
    loan = db.execute_query(f"FIND loan with {COMMENT.name}='{TESTLOANCOMMENT}'", unique=True)
    assert loan.get_property(f"{LOAN_ACCEPTED.name}").value.startswith("20")
    assert loan.get_property(f"{LENT.name}") is None
    assert loan.get_property(f"{RETURN_REQUESTED.name}") is None

    ##### confirm loan #####
    data = {}
    data[F_LOAN] = loan.id
    response = run_server_side_script("loan_management/confirm_loan.py",
                                    #"pos0",
                                    #option1="val1",
                                    files={"-p0": save_dict_to_jsonfile(data)},
                                    **{"auth-token":db.get_connection()._authenticator.auth_token})
    assert response.stderr is None
    assert response.code == 0
    loan = db.execute_query(f"FIND loan with {COMMENT.name}='{TESTLOANCOMMENT}'", unique=True)
    assert loan.get_property(f"{LENT.name}").value.startswith("20")

    ##### request return #####
    data = {}
    data[F_LOAN] = loan.id
    # TODO: this is writen in the content field. Change name?
    data[F_COMMENT] = TESTRETURNCOMMENT
    # TODO: this is written in RETURNLOCATION. Change name?
    data[F_CURRENT_LOCATION]=db.utils.get_entity.get_entity_by_name(TESTCURRENTDESTINATION).id
    data[F_EXPECTED_RETURN_DATE] = TESTRETURNDATE
    data[F_EMAIL]=TESTBORROWEREMAIL
    data[F_LAST_NAME]=TESTLN
    data[F_FIRST_NAME]=TESTFN
    response = run_server_side_script("loan_management/request_return.py",
                                    #"pos0",
                                    #option1="val1",
                                    files={"-p0": save_dict_to_jsonfile(data)},
                                    **{"auth-token":db.get_connection()._authenticator.auth_token})
    assert response.stderr is None, response.stderr
    assert response.code == 0
    loan = db.execute_query(f"FIND loan with {COMMENT.name}='{TESTLOANCOMMENT}'", unique=True)
    assert loan.get_property(f"{LOAN_ACCEPTED.name}").value.startswith("20")
    assert loan.get_property(f"{CONTENT.name}").value == TESTRETURNCOMMENT
    assert loan.get_property(f"{EXPECTED_RETURN.name}").value == TESTRETURNDATE
    assert loan.get_property(f"{RETURNLOCATION.name}").value == db.utils.get_entity.get_entity_by_name(TESTCURRENTDESTINATION).id
    assert loan.get_property(f"{RETURN_REQUESTED.name}").value.startswith("20")
    assert loan.get_property(f"{RETURN_ACCEPTED.name}") is None
    # TODO test change of borrower

    # accept return
    data = {}
    data[F_LOAN] = loan.id
    response = run_server_side_script("loan_management/accept_return_request.py",
                                    #"pos0",
                                    #option1="val1",
                                    files={"-p0": save_dict_to_jsonfile(data)},
                                    **{"auth-token":db.get_connection()._authenticator.auth_token})
    assert response.stderr is None, response.stderr
    assert response.code == 0
    loan = db.execute_query(f"FIND loan with {COMMENT.name}='{TESTLOANCOMMENT}'", unique=True)
    assert loan.get_property(f"{RETURN_ACCEPTED.name}").value.startswith("20")
    assert loan.get_property(f"{RETURNED.name}") is None

    # manual return
    data = {}
    data[F_LOAN] = loan.id
    response = run_server_side_script("loan_management/manual_return.py",
                                    #"pos0",
                                    #option1="val1",
                                    files={"-p0": save_dict_to_jsonfile(data)},
                                    **{"auth-token":db.get_connection()._authenticator.auth_token})
    assert response.stderr is None
    assert response.code == 0
    loan = db.execute_query(f"FIND loan with {COMMENT.name}='{TESTLOANCOMMENT}'", unique=True)
    assert loan.get_property(f"{RETURNED.name}").value.startswith("20")
